/*
Temperature in a metal rod : 

- the temperature is imposed at one end (surface SV) 
- convection happens on all other surfaces (surface SA) (j.n = h(T - T_ext))
- conduction of heat in the material (volume D) (j = - lambda Grad(T))
- heat radiation on surface SA (j.n = a sigma (T^4 - T_ext^4))
- equilibrium situation (Div(j)=0) 

*/

T_ext = 30.;		// (°C) outside air temperature
T_chaud = 500.;		// (°C) hot temperature on surface SV
T_K = 273.;		// Conversion K <-> °C

h = 30.;   // Wm-2K-1 (convection constant for strong natural air convection)
lambda = 20.;   // Wm-1K-1 (conductivity for some highly allied steel)
a = 0.9;	// emissivity of the surface
sigma = 5.7e-8; // Wm-2K-4 (Stefan constant for thermal radiation)

Group{
	D = Region[33];		// Entire volume 
	SV = Region[31];	// Surface with imposed temperature
	SA = Region[32];	// Other surfaces with air convection
	S = Region[{SV,SA}];	// All surfaces
}


Function {
}


// Temperature T_chaud imposed on surface SV 
Constraint {
	{ Name contraintes_temperature ; 
	  Case {
	    { Region SV ; Type Assign ; Value T_chaud ; }
	    { Region D ; Type Init ; Value T_ext ;}
	  }
  	}

}

FunctionSpace {
	{ Name temperature_dans_barre ; Type Form0 ;
	  BasisFunction {
	  	{ Name w_n ; NameOfCoef T_n ; Function BF_Node ;
		  Support Region[{D,SA}] ; Entity NodesOf[ All ] ; }
	  }
	  Constraint {
	  	{ NameOfCoef T_n ; EntityType NodesOf ; 
		  NameOfConstraint contraintes_temperature ; }
	  }
	}
}

Jacobian {
  { Name JVol ;
    Case { 
      { Region All ; Jacobian Vol ; }
    }
  }
  { Name JSur ;
    Case { 
      { Region All ; Jacobian Sur ; }
    }
  }
}

Integration {
  { Name I ;
    Case { 
      { Type Gauss ;
        Case { 
	  { GeoElement Point       ; NumberOfPoints  1 ; }
          { GeoElement Line        ; NumberOfPoints  3 ; }
          { GeoElement Triangle    ; NumberOfPoints  4 ; }
          { GeoElement Quadrangle  ; NumberOfPoints  4 ; }
          { GeoElement Tetrahedron ; NumberOfPoints  4 ; }
          { GeoElement Hexahedron  ; NumberOfPoints  6 ; }
          { GeoElement Prism       ; NumberOfPoints  6 ; } 
        }
      }
    }
  }
}

/****************** Integral formulation to solve :
find T such that :

\int_D lambda Grad(T).Grad(T') + \int_SA h (T - T_ext) T' + ...
+ \int_SA a sigma (T^4 - T_ext^4) T' = 0
holds for all T' with T' = 0 on SV
******************************************/

Formulation {
  { Name formulation_du_probleme ; Type FemEquation ;
    Quantity { 
      { Name T ; Type Local ; NameOfSpace temperature_dans_barre ; }
    }
    Equation {
      Galerkin { [ lambda * Dof{d T} , {d T} ] ;  
                 In D ; Jacobian JVol ; Integration I ; }

      Galerkin { [ h * Dof{T} , {T} ] ;
                 In SA ; Jacobian JSur ; Integration I ; }

      Galerkin { [ h * (- T_ext) , {T} ] ;
                 In SA ; Jacobian JSur ; Integration I ; }

      Galerkin { [ a * sigma * ({T} + T_K)^4 , {T} ] ;
                 In SA ; Jacobian JSur ; Integration I ; }
/* 
// La ligne précédente peut être remplacée par ce qui suit 
// pour accélérer la convergence :
   Galerkin { [ a * sigma * Dof{T} * ({T} + T_K)^3 , {T} ] ;
                 In SA ; Jacobian JSur ; Integration I ; }

   Galerkin { [ a * sigma * T_K * ({T} + T_K)^3 , {T} ] ;
                 In SA ; Jacobian JSur ; Integration I ; }
*/

      Galerkin { [ - a * sigma * (T_ext + T_K)^4 , {T} ] ;
                 In SA ; Jacobian JSur ; Integration I ; }

   }
  }
}

// Parameters for the iterative loop :
relaxation = 1.0;
error_max = 1e-3;
max_loops = 50;

Resolution {
  { Name resolution_pb_lineaire ;
    System {
      { Name A ; NameOfFormulation formulation_du_probleme ; }
    }
    Operation { 
      InitSolution[A];
      Generate[A] ; 
      Solve[A] ; 
      SaveSolution[A] ; 
    }
  }

  { Name resolution_pb_nonlineaire ;
    System {
      { Name A ; NameOfFormulation formulation_du_probleme ; }
    }
    Operation {
      InitSolution[A];
      IterativeLoop[max_loops, error_max, relaxation] {
		GenerateJac[A] ; 
      		SolveJac[A] ;
//		SaveSolution[A];
      } 
      SaveSolution[A] ; 
    }
  }

}

PostProcessing {
  { Name post_processing ; NameOfFormulation formulation_du_probleme ;
    Quantity {
      { Name T; Value { Local { [ {T} ] ; In D ; Jacobian JVol ; }}}
/*      { Name TS; Value { Local { [ {T} ] ; In S ; Jacobian JSur ; }}}
      { Name P; Value { Local { [ h[] * ({T} - T_ext[]) * Normal[] ] ; 
	In SA ; Jacobian JSur ;}}}

      { Name j; Value { Local { [ - lambda * {d T} ] ;
        In D ; Jacobian JVol ;}}}
      { Name nj; Value { Local { [ Norm [ - lambda[] * {d T}] ] ;
        In D ; Jacobian JVol ;}}}


// Intégrale d'une quantité sur chaque facette :
      { Name Out_SA; Value { Integral { [ h[] * ({T} - T_ext[]) ] ; 
        In SA ; Integration I ; Jacobian JSur ; }}}

// Essai :
      { Name Out_SA2; Value { Local { [ h[] * ({T} - T_ext[]) ] ; 
        In SA ; Jacobian JSur ; }}}


      { Name In_SV; Value { Integral { [ - lambda[] * {d T} ] ; 
        In SV ; Integration I ; Jacobian JSur ; }}}
*/
    }
  }
}

PostOperation {
  { Name post_operations ; NameOfPostProcessing post_processing;
    Operation {
	Print[ T, OnElementsOf D, File "T.pos" ] ;
//    Print[ TS, OnElementsOf S, File "T1.pos"] ;
//    Print[ T, OnSection {{0,0,0}{0,0,1}{1,0,0}} , File "T2.pos"] ;
//    Print[ T, OnSection {{1,0,0.02}{0,1,0.02}{0,0,0.02}}, File "T3.pos"] ;
//   Print[ P, OnElementsOf SA, File "T2.pos"] ;
//   Print[ nj, OnElementsOf D, File "T2.pos", Format Table] ;
//   Print[ j, OnElementsOf D, File "T2.pos"] ;
// Print[ Out_SA, OnElementsOf SA, File "T2.pos"] ;
// Print[ In_SV, OnElementsOf SV, File "T3.pos"] ;
// Print [ Out_SA, OnRegion SA, Store RES ] ;
    }
  }
}

